# Before using the software, run 
#   R CMD SHLIB const2.cpp
# to compile the c code that will be used by R program.

# The following is an example of running BGM analysis on an example data

# Clear the workspace
rm(list = ls());

# Load BGM R functions and c library
source("./Functions.R");

# Load example data of A1CF and A2M. There are 6 features/nodes and 1448 samples.
load("exampleData.rda");

# Generate a list called Input to be input into the NetworkAnalysis function for doing BGM analysis
#Input = list(y = data,  # y is the input data, each row is a node in the network #and each column is a sample 
#  Rep = 20000,
#  graph.prior = rbind(c(0, 1, 0, 0, 0, 0), c(1, 0, 1, 0, 0, 0), c(0, 1, 0, 0, 0, #0), 
#                      c(0, 0, 0, 0, 1, 0), c(0, 0, 0, 1, 0, 1), c(0, 0, 0, 0, 1, #0)), 
             # graph.prior is a symmetric 0,1 matrix representing the prior #network used to guide model training, 
             # where 1 indicates an interaction and 0 indicates no interaction.
#  pho = 1); # pho is the pho value used to control the strength of prior network
  
  
Input = list(y = data,  # y is the input data, each row is a node in the network and each column is a sample 
  Rep = 20000,
  graph.prior = diag(4), 
             # graph.prior is a symmetric 0,1 matrix representing the prior network used to guide model training, 
             # where 1 indicates an interaction and 0 indicates no interaction.
  pho = 1); # pho is the pho value used to control the strength of prior network

Result = NetworkAnalysis(Input);
save(Result, "Nnode4.RData")