#include <stdio.h>
#include <R.h>
#include <Rmath.h>
#include <Rembedded.h>
#include "NConstant.h"
#include "MCMCratio.h"
#include "update_z.h"
#include "update_e.h"
#include "update_p.h"
#include "update_mu.h"
#include "update_sigma.h"
#include "update_kminus.h"
#include "update_kplus.h"
#include "update_beta.h"
#include "update_graph.h"
#include <Rdefines.h>
#include <assert.h>
#include <math.h>
#include <cstdio>
#include <vector>

extern "C"
{
//  const int N = 10000;

  // Gibbs function:
  void Gibbs(double *yvector, double *betavector, int *graphvector, int *evector, int *zvector, double *p, double *mu, double *sigma, double *kminus, double *kplus, double *inibetavector, int *inigraphvector, int *graphpriorvector, int &Nnode, int &Nsample, double &pho, const int &N)
  {
    int c1, c2, c3 = Nnode*Nnode, c4, i, j, k, n;
	  double ***beta;
	  int ***graph;
	  beta = new double** [Nnode];
	  graph = new int** [Nnode];
	  for (i=0; i<Nnode; i++)
	  {
		  beta[i] = new double* [Nnode];
		  graph[i] = new int* [Nnode];
		  for (j=0; j<Nnode; j++)
		  {
			  beta[i][j] = new double[N];
			  graph[i][j] = new int[N];
		  }
	  }
		  
    
    // Transdimenal variables:
	  double **y;
	  int **z;
	  int **e;
	  double **betaold;
	  double **betanew;
	  int **graphold;
	  int **graphnew;
	  
	  y = new double* [Nnode];
	  z = new int* [Nnode];
	  e = new int* [Nnode];
	  betaold = new double* [Nnode];
	  betanew = new double* [Nnode];
	  graphold = new int* [Nnode];
	  graphnew = new int* [Nnode];
	
	  for (i=0; i<Nnode; i++)
	  {
		  y[i] = new double[Nsample];
		  z[i] = new int[Nsample];
		  e[i] = new int[Nsample];
		  betaold[i] = new double[Nnode];
		  betanew[i] = new double[Nnode];
		  graphold[i] = new int[Nnode];
		  graphnew[i] = new int[Nnode];
	  }
	  
	
    
    // Initialize:
    for (i=0; i<Nnode; i++)
      for (j=0; j<Nsample; j++)
	{
	  c1 = Nnode*j+i;
	  e[i][j]=evector[c1];
	  z[i][j]=zvector[c1];
	  y[i][j]=yvector[c1];
	}
    
    for (i=0; i<Nnode; i++)
      for (j=0; j<Nnode; j++)
	for (k=0; k<N; k++)
	  {
	    c1 = Nnode*j+i;   
	    beta[i][j][k]=inibetavector[c1];
	    graph[i][j][k]=inigraphvector[c1];						   
	  }
    
    
    // Start:
    for (n=1; n<N; n++)
      {
		  for (i=0; i<Nnode; i++)
		  {
			  for (j=0; j<Nnode; j++)
			  {
				  betaold[i][j] = beta[i][j][n-1];
				  graphold[i][j] = graph[i][j][n-1];
			  }
		  }
	
	update_z(y, betaold, graphold, p, mu, sigma, kminus, kplus, z, Nnode, Nsample);
	update_e(y, betaold, graphold, p, mu, sigma, kplus, z, e, Nnode, Nsample);
	update_p(e, p, Nnode, Nsample);
	update_mu(y, sigma, e, kminus, kplus, mu, Nnode, Nsample);
	update_sigma(y, mu, e, sigma, Nnode, Nsample);
	update_kminus(y, mu, e, kminus, Nnode, Nsample);
	update_kplus(y, mu, e, kplus, Nnode, Nsample);	
	for (i=0; i<Nnode; i++)
	  for (j=0; j<Nnode; j++)
	    betanew[i][j] = beta[i][j][n]; 

	update_beta(betaold, betanew, z, graphold, Nnode, Nsample);
	
		  for (i=0; i<Nnode; i++){
	  for (j=0; j<Nnode; j++)
	    {
	      beta[i][j][n] = betanew[i][j];
	      graphnew[i][j] = graph[i][j][n];
	    }
		  }	
	update_graph(z, graphold, graphnew, betanew, graphpriorvector, Nnode, Nsample, pho);	
	
		  for (i=0; i<Nnode; i++){
	  for (j=0; j<Nnode; j++)
	    {
	      beta[i][j][n] = betanew[i][j];
	      graph[i][j][n] = graphnew[i][j];
	    }}
      }
    
	  for (i=0; i<Nnode; i++)
	  {
		  for (j=0; j<Nnode; j++)
		  {
			  c4 = Nnode*j+i;
			  for (k=0; k<N; k++)
			  {
				  c2 = c3*k+c4;
	              betavector[c2] = beta[i][j][k];
	              graphvector[c2] = graph[i][j][k];
			  }
		  }
	  }
	  
	  ///Delete the memory
	  for (i=0; i<Nnode; i++)
	  {
		  delete[] graphnew[i]; graphnew[i]=NULL;
		  delete[] graphold[i]; graphold[i]=NULL;
		  delete[] betanew[i]; betanew[i]=NULL;
		  delete[] betaold[i]; betaold[i]=NULL;
		  delete[] e[i]; y[i]=NULL;
		  delete[] z[i]; y[i]=NULL;
		  delete[] y[i]; y[i]=NULL;
	  }
	  delete[] graphnew; graphnew = NULL;
	  delete[] graphold; graphold = NULL;
	  delete[] betanew; graphnew = NULL;
	  delete[] betaold; graphold = NULL;
	  delete[] e; e = NULL;
	  delete[] z; z = NULL;
	  delete[] y; y = NULL;
	  for (i=0; i<Nnode; i++)
	  {
		  for (j=0; j<Nnode; j++)
		  {
			  delete[] graph[i][j]; graph[i][j]=NULL;
			  delete[] beta[i][j]; beta[i][j]=NULL;
		  }
		  delete[] graph[i]; graph[i]=NULL;
		  delete[] beta[i]; beta[i]=NULL;
	  }
	  delete[] graph; graph = NULL;
	  delete[] beta; beta = NULL;
	  return;
	  
  }
}

