rm(list=ls(all=TRUE))
Nnode = 5 # number of nodes
T = 300  # number of samples
library(mc2d)  # function 'rtrunc' is contained
library(MASS)  # function 'mvrnorm' is contained
library(sfsmisc)  #for digitsBase
source("initialfunction.r")
source("Gibbsfunction.r")
source("update_post.r")
#dyn.load("const.so")
seed=1
set.seed(seed)


#####################

## hyperparameters ##

#####################
tau.alpha = 0.1

theta.mu = rep(0,Nnode);

tau.mu = rep(1,Nnode)

gamma.sigma = rep(2,Nnode); lambda.sigma = rep(0.1,Nnode)

gamma.kplus= rep(11,Nnode)
gamma.kminus = rep(11,Nnode)

lambda.kplus = rep(40,Nnode)
lambda.kminus = rep(40,Nnode)
sigma.beta = 3 # sd for prior normal of beta
sigma.betapro = 0.3   # sd for proposing new beta when updating GRAPH



#######################

## define parameters ##

#######################

graph = matrix(0,Nnode,Nnode)  # graph

beta = matrix(0,Nnode,Nnode)  # beta

z = matrix(0,Nnode,T)  # z1 is zD, z2 is zR, z3 is zM, the three expression level inmicator, takes (0,1)

e = matrix(0,Nnode,T)  # inmicator, takes (-1,0,1), z=0 <-> e=0

p = rep(0,Nnode)  # probabilities p[i] = P(e=-1|z=1)

y = rep(Nnode,T)


###########################################################################

## 1. functions for sample parameters   

###########################################################################



##(1) sample graph

graph.sample <- function()

{

u = runif(Nnode^2,0,1)

va = 1*(u>0.5)

graph=matrix(va,Nnode,Nnode)

for (i in 1:(Nnode-1)){
  for (j in (i+1):Nnode){
    graph[j,i]<-graph[i,j]
  }}

return(graph)

}



##(2) sample beta's

beta.sample <- function(graph)

{

beta = matrix(0,Nnode,Nnode)

diag(beta) = rnorm(Nnode,sample(c(-0.4,.4),1),.5)

for (i in 1:(Nnode-1)){
  for (j in (i+1):Nnode){
    beta[i,j]<-(graph[i,j]==1)*rnorm(1, sample(c(-4,4),1), 0.5)
    beta[j,i]=beta[i,j]
  }}
return(beta)
}



##(3) generate z's

z.sample <- function(graph,beta)

{
z = matrix(0,Nnode,T)
prob = z.prob(graph,beta)
index = sample(1:2^Nnode, T, prob=prob, replace=T)  # pick out the index
combina=digitsBase(1:2^Nnode-1)
z=combina[,index]
return(z)
}



##(4) compute prob's for inmicator vector z

zt.prob <- function(beta,zt)

{beta1<-diag(beta)
v = exp(beta1)/(1+exp(beta1))
prob.part2<-0
for (i in 1:Nnode)
  {prob.part2<-prob.part2+sum(beta[i,-i]*(zt[i]-v[i])*(zt[-i]-v[-i]))}
prob=exp(sum(beta1*zt)+prob.part2/2)
return(prob)
}

z.prob <- function(graph,beta)

{
  prob = rep(0,2^Nnode)  # for each z, there are two choices 0 anm 1, so a total of 8 combinations

combina=digitsBase(1:2^Nnode-1)
for (i in 1:2^Nnode)
  {
    prob[i]<-zt.prob(beta,combina[,i])  }

prob = prob/(sum(prob))

return(prob)

}



##(5) generate p's for e

p.sample <- function()

{

p = runif(Nnode,0.2,0.8)

p = rep(p, T)

p = matrix(p,Nnode,T)

return(p)

}



##(6) generate e's

e.sample <- function(z,p)

{

e = matrix(0,Nnode,T)

u = runif(Nnode*T,0,1)

u = matrix(u,Nnode,T)

e[z==0] = -1

e[z==1] = ifelse(u[z==1]<p[z==1],0,1)

return(e)

}



##(7) generate theta

theta.sample <- function()

{

mu = theta.mu

sigma = sqrt(lambda.sigma/(gamma.sigma-1))

kminus = lambda.kminus/(gamma.kminus-1)

kplus = lambda.kplus/(gamma.kplus-1)

return(list(mu = mu, sigma = sigma, kminus = kminus, kplus = kplus))

}


##########################################################################

## 2. sample responses

##########################################################################
y.sample<-function(e,theta)

{
  y = matrix(0,Nnode,T)
  mu = theta$mu; sigma = theta$sigma; kminus = theta$kminus; kplus = theta$kplus
  for (i in 1:Nnode){
  y[i,e[i,]==-1] = runif(sum(e[i,]==-1),mu[i] - kminus[i], mu[i])
  y[i,e[i,]==0] = rnorm(sum(e[i,]==0),mu[i], sigma[i])
  y[i,e[i,]==1] = runif(sum(e[i,]==1),mu[i], mu[i]+kplus[i])}
  return(y)
}
##(1) generate parameters

graph = graph.sample()
#graph = matrix(c(1,1,0,1,1,1,1,0,0,1,1,1,1,0,1,1),ncol=4)

beta = beta.sample(graph)

z = z.sample(graph,beta)

p = p.sample()

e = e.sample(z,p)

apply(e==-1,1,sum)

apply(e==1,1,sum)

theta = theta.sample()




##(2) generate responses

y = y.sample(e, theta)

y = t(scale(t(y)))

data = y

source("./Functions.R");

Input = list(y = data,  # y is the input data, each row is a node in the network and each column is a sample 
  Rep = 20000,
  graph.prior = diag(Nnode),
             # graph.prior is a symmetric 0,1 matrix representing the prior network used to guide model training, 
             # where 1 indicates an interaction and 0 indicates no interaction.
  pho = 1); # pho is the pho value used to control the strength of prior network

Result = NetworkAnalysis(Input);
save(Result, file=paste("Nnode",Nnode,"seed",seed,".RData",sep=""))
